import numpy as np

import argparse

import matplotlib.pyplot as plt

BP_FILE_NAME = ""
PC_FILE_NAME = ""
OUTFILE_NAME = ""
TITLE = "My Reward Plot"
X_LABEL = "episode"
INCREMENTS = 1


def compute_confidence_intervals(matrix):
    upper_confidence_interval = []
    lower_confidence_interval = []

    for iteration in matrix.transpose():
        bootstrapped_means = []
        for B in range(2000):
            bootstrapped_sample = np.random.choice(iteration, len(iteration))
            bootstrapped_means.append(np.mean(bootstrapped_sample))
        bootstrapped_percentiles = np.percentile(np.array(bootstrapped_means), [2.5, 97.5])
        upper_confidence_interval.append(bootstrapped_percentiles[1])
        lower_confidence_interval.append(bootstrapped_percentiles[0])

    return upper_confidence_interval, lower_confidence_interval

def plot(bp_reward_matrix, pc_reward_matrix, confidence_intervals=False):      

    bp_reward_matrix_mean = np.mean(bp_reward_matrix, axis=0)
    pc_reward_matrix_mean = np.mean(pc_reward_matrix, axis=0)

    plt.style.use('ggplot')
    plt.figure(figsize=(10, 5))
    plt.title(TITLE)
    
    plt.plot([x * INCREMENTS for x in range(bp_reward_matrix.shape[1])], bp_reward_matrix_mean, linewidth=2, color='blueviolet', alpha=0.8, label="BP")
    plt.plot([x * INCREMENTS for x in range(pc_reward_matrix.shape[1])], pc_reward_matrix_mean, linewidth=2, color='firebrick', alpha=0.8, label="PC")
    plt.legend()

    if confidence_intervals:
        bp_upper_confidence_interval, bp_lower_confidence_interval = compute_confidence_intervals(bp_reward_matrix)
        pc_upper_confidence_interval, pc_lower_confidence_interval = compute_confidence_intervals(pc_reward_matrix)
        plt.fill_between([x * INCREMENTS for x in range(bp_reward_matrix.shape[1])], bp_upper_confidence_interval, bp_lower_confidence_interval, color='blueviolet', alpha=0.4)
        plt.fill_between([x * INCREMENTS for x in range(pc_reward_matrix.shape[1])], pc_upper_confidence_interval, pc_lower_confidence_interval, color='firebrick', alpha=0.4)

    plt.xlabel(X_LABEL)
    plt.ylabel('reward')
    plt.savefig(OUTFILE_NAME)
    plt.close('all')


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--bp-file", default=BP_FILE_NAME)
    parser.add_argument("--pc-file", default=PC_FILE_NAME)
    parser.add_argument('-o', "--out-file", default=OUTFILE_NAME)
    parser.add_argument('-t', "--title", default=TITLE)
    parser.add_argument('-x', "--x-label", default=X_LABEL)
    parser.add_argument('-i', "--increments", default=INCREMENTS)
    args = parser.parse_args()

    BP_FILE_NAME = args.bp_file
    PC_FILE_NAME = args.pc_file
    OUTFILE_NAME = args.out_file
    TITLE = args.title
    X_LABEL = args.x_label
    INCREMENTS = args.increments

    with open(BP_FILE_NAME, 'rb') as infile:
        bp_reward_matrix = np.load(infile)

    with open(PC_FILE_NAME, 'rb') as infile:
        pc_reward_matrix = np.load(infile)
    
    plot(bp_reward_matrix, pc_reward_matrix, (bp_reward_matrix.shape[0] >= 3 and pc_reward_matrix.shape[0] >= 3))